<?php
/* 
Plugin Name: WPMU Global Search
Plugin URI: http://mambanegra86.wordpress.com
Description: Adds the ability to search through blogs into your WordPress MU installation.
Version: 1.1
Author: Alicia García Holgado
*/

load_plugin_textdomain('wpmu-global-search', MUPLUGINDIR.'/wpmu-global-search/langs');

include_once('includes/functions.php');

function wpmu_globalsearch($myargs = '') {
    if (is_array($myargs)) $args = &$myargs;
    else parse_str($myargs, $args);

    $defaults = array('before_widget'=>'','after_widget'=>'',
    'before_title'=>'<h2>','after_title'=>'</h2>'
    );
    $args = array_merge($defaults, $args);

    widget_wp_wpmu_globalsearch($args);
}

function widget_wp_wpmu_globalsearch($args) {
    extract($args);
    $options = get_option('widget_wp_wpmu_globalsearch');
    $title = empty($options['title']) ? __( 'Global Search', 'wpmu-global-search' ) : attribute_escape($options['title']);
    $page = empty($options['page']) ? __( 'globalsearch', 'wpmu-global-search' ) : attribute_escape($options['page']);

    echo $before_widget . $before_title . $title . $after_title;
    wpmu_globalsearch_form($page);
    echo $after_widget;
}

function widget_wp_wpmu_globalsearch_control() {
    $options = $newoptions = get_option('widget_wp_wpmu_globalsearch');
    if ( $_POST['wpmusearch-submit'] ) {
        $newoptions['title'] = strip_tags(stripslashes( $_POST['wpmusearch-title']));
        $newoptions['page'] = strip_tags(stripslashes( $_POST['wpmusearch-page']));
    }
    if ( $options != $newoptions ) {
        $options = $newoptions;
        update_option( 'widget_wp_wpmu_globalsearch', $options );

    }
    $title = attribute_escape($options['title']);
    $page = attribute_escape($options['page']);

    echo '<p><label for="wpmusearch-title">';
    _e( 'Title:', 'wpmu-global-search' );
    echo '<input class="widefat" id="wpmusearch-title" name="wpmusearch-title" type="text" value="'.$title.'" /></label></p>'
    .'<p><label for="wpmusearch-page">';
    _e( 'Search page:', 'wpmu-global-search' );
    echo '<input class="widefat" id="wpmusearch-page" name="wpmusearch-page" type="text" value="'.$page.'" /></label></p>'
    .'<input type="hidden" id="wpmusearch-submit" name="wpmusearch-submit" class="button" value="1" />';
}

function widget_wp_wpmu_globalsearch_init() {
    if ( !function_exists('register_sidebar_widget') ) return;
    // Register widget for use
    register_sidebar_widget(array('WPMU Global Search', 'widgets'), 'widget_wp_wpmu_globalsearch');
    register_widget_control(array('WPMU Global Search', 'widgets'), 'widget_wp_wpmu_globalsearch_control');
}

function wp_wpmu_globalsearch_css() {
    $myStyleFile = WPMU_PLUGIN_URL . '/wpmu-global-search/style.css';
    wp_register_style('wp_wpmu_globalsearch_css_styles', $myStyleFile);
    wp_enqueue_style( 'wp_wpmu_globalsearch_css_styles');
}

function wp_wpmu_globalsearch_init() {
    global $wp;

    if(isset($_GET['uws'])) {
        $wp->add_query_var('uws');
        $wp->set_query_var('uws', $_GET['uws']);
    }
}

add_shortcode('page_wpmu_search', 'wp_wpmu_globalsearch_page');
function wp_wpmu_globalsearch_page($atts) {
    return wpmu_globalsearch_page();
}

/*
  Builds a view that contains posts from all blogs.
  This function will only run when a new blog is created, deleted, archived, or suspended.
*/
function wpmu_globalsearch_build_views() {
    global $wp_query, $wpdb;

    $union_queries = wpmu_globalsearch_build_v_query();

    $v_post_select_query     = $union_queries['posts'];
    $v_postmeta_select_query = $union_queries['postmeta'];
    $v_comments_select_query = $union_queries['comments'];

    $v_query  = "CREATE OR REPLACE VIEW `{$wpdb->base_prefix}v_posts` AS ";
    $v_query .= $v_post_select_query;
    $wpdb->query($v_query);

    $v_query  = "CREATE OR REPLACE VIEW `{$wpdb->base_prefix}v_postmeta` AS ";
    $v_query .= $v_postmeta_select_query;
    $wpdb->query($v_query);

    $v_query  = "CREATE OR REPLACE VIEW `{$wpdb->base_prefix}v_comments` AS ";
    $v_query .= $v_comments_select_query;
    $wpdb->query($v_query);

}

function wpmu_globalsearch_build_v_query() {
    global $wpdb;

    $blogs = get_blog_list(0, 'all');
    $i = 0;
    $v_query = '';
    foreach ($blogs as $blog) {
        if ($i != 0) {
            $posts_select_query    .= ' UNION ';
            $postmeta_select_query .= ' UNION ';
            $comments_select_query .= ' UNION ';
        }
        $posts_select_query    .= " (SELECT '{$blog['blog_id']}' AS blog_id, '{$blog['domain']}' AS domain, '{$blog['path']}' AS path, posts{$blog['blog_id']}.* FROM {$wpdb->base_prefix}{$blog['blog_id']}_posts posts{$blog['blog_id']}) ";
        $postmeta_select_query .= " (SELECT '{$blog['blog_id']}' AS blog_id, '{$blog['domain']}' AS domain, '{$blog['path']}' AS path, postmeta{$blog['blog_id']}.* FROM {$wpdb->base_prefix}{$blog['blog_id']}_postmeta postmeta{$blog['blog_id']}) ";
        $comments_select_query .= " (SELECT '{$blog['blog_id']}' AS blog_id, '{$blog['domain']}' AS domain, '{$blog['path']}' AS path, comments{$blog['blog_id']}.* FROM {$wpdb->base_prefix}{$blog['blog_id']}_comments comments{$blog['blog_id']}) ";
        $i++;
    }
    $query_array = array(
        'posts'    => $posts_select_query,
        'postmeta' => $postmeta_select_query,
        'comments' => $comments_select_query
    );

    return $query_array;
}

function wpmu_globalsearch_page() {
    global $wp_query, $wpdb;

    if(!empty($wp_query->query_vars['uws'])) {
        $term = $wp_query->query_vars['uws'];
        $request = "SELECT {$wpdb->base_prefix}v_posts.* from {$wpdb->base_prefix}v_posts left join {$wpdb->base_prefix}users on {$wpdb->base_prefix}users.ID={$wpdb->base_prefix}v_posts.post_author ".
		"where (post_title LIKE '%$term%' OR post_content LIKE '%$term%' OR {$wpdb->base_prefix}users.display_name LIKE '%$term%')".
        "AND (post_status = 'publish' OR post_status = 'private') AND post_type = 'post' AND blog_id != '1' ORDER BY {$wpdb->base_prefix}v_posts.comment_count DESC, post_date DESC";
        $search = $wpdb->get_results($request);

		$author = "SELECT * from {$wpdb->base_prefix}v_posts ";
        if(empty($search)) {
            echo "<h2 class='center'>".__("Not found", 'wpmu-global-search')."</h2>".
                "<p class='center'>".__("Sorry, but you are looking for something that isn't her", 'wpmu-global-search')."</p>";
        } else {
            $blogid = '';
            foreach($search as $s) {
                $author = get_userdata($s->post_author);
                if($blogid != $s->blog_id) {
                    $blogid = $s->blog_id;
                    echo "<h2 class='searchblog'>". get_blog_option($blogid, 'blogname') ."</h2>";
                }?>

                <div <?php post_class() ?>>
                    <h2 id="post-<?php echo $s->ID.$s->blog_id; ?>" class="searchtitle"><a href="<?php echo get_blog_permalink( $s->blog_id, $s->ID ); ?>" rel="bookmark" title="Permanent Link to <?php echo $s->post_title ?>"><?php echo $s->post_title ?></a></h2>
                    <small class="date"><?php echo date('j/m/y, G:i' ,strtotime($s->post_date)); ?></small>

                    <div class="act">
                        <small class="authorpost"><?php echo '<a href="http://' . $s->domain.$s->path.'author/'.$author->user_nicename . '" title="' . $author->user_nicename . '">' . $author->user_nicename . '</a>'; ?></small>
                        <?php echo wpmu_globalsearch_get_edit_link($s, 'Editar', '<small class="editpost">', '</small>'); ?>
                    </div>

                    <div class="entry">
                        <?php echo wpmu_globalsearch_get_the_content($s); ?>
                    </div>

                    <div class="postmetadatasearch">
                        <small class="comments">
                            <?php wpmu_globalsearch_get_comments_link($s, 'Sin comentarios', '1 Comentario', '% Comentarios'); ?>
                        </small>
                    </div>
                </div>
            <?php
            }
        }
    } else {
        echo "<h2 class='center'>".__("Not found", 'wpmu-global-search')."</h2>".
            "<p class='center'>".__("Sorry, but you are looking for something that isn't her", 'wpmu-global-search')."</p>";
    }
}

// Run code and init
add_action ('wpmu_new_blog', 'wpmu_globalsearch_build_views');
add_action ('wpmuadminedit', 'wpmu_globalsearch_build_views');

add_action('wp_print_styles', 'wp_wpmu_globalsearch_css');
add_action('init', 'wp_wpmu_globalsearch_init');
add_action('widgets_init', 'widget_wp_wpmu_globalsearch_init');

?>
